!
!	Hi-Speed B_PUTC routine
!

		.define	blink
		.define	_change_screen
		.define	_move_to
		.define	_put_c
!
!	I/O address etc...
!

TIMERC		=	0x110		! interrupt vector address from Timer-C

TVRAM0		=	0xe00000	! plane0 addr. of text VRAM
TVRAM1		=	TVRAM0 + 0x80 * 512
					! plane1 addr. of text VRAM

CRTCR11		=	0xe80016	! CRTC register
CRTCR21		=	0xe8002a
CRTCR22		=	0xe8002c
CRTCR23		=	0xe8002e
CRTCCTRL	=	0xe80480

MFPGPIP		=	0xe88001	! MFP GPIP data register

CGROM8x16	=	0xf3a800	! 8x16 font addr.
CGROM16x16	=	0xf00000	! 16x16 font addr.
_FNTADR		=	0x0016

BLINK_COUNT	=	100
ALLPLANE	=	15

MAXX		=	96-1
MAXY		=	32-1

!
!	OFFSET
!

XPOS		=	 0
YPOS		=	 2
REVERSE		=	 4
MASK		=	 6
KFLAG		=	 8
ESC_STATE	=	10
ESC_PARA_NUM	=	12
ESC_PARAMETER	=	14

		.sect	.data
!
!	work area
!

cursor_stat:	.data2	1	! 0 : cursor invisible, else visible
blink_stat:	.data2	0	! 0 : now cursor OFF,  else ... ON
blink_count:	.data2	BLINK_COUNT

!plane:		.data2	0	! now accessable plane
vplane:		.data2	0	! now visible plane 0:plane0, else plane1

plane0:		.data2	0,0,0,ALLPLANE,0,0,0,0,0,0,0,0,0,0,0
plane1:		.data2	0,0,0,ALLPLANE,0,0,0,0,0,0,0,0,0,0,0

		.sect	.text

!
!	interruput routine
!

blink:
	move.w	sr,-(sp)
	move.w	#0x2700,sr	! disable interrupt
	movem.l	d0/a0,-(sp)

	move.w	cursor_stat,d0
	bne	blinking	! cursor visible
	move.w	blink_stat,d0
	bne	blinking	! invisible and now curosor on ---> erase it
	bra	bl_quit

blinking:
	lea	blink_count,a0
	sub.w	#1,(a0)
	beq	blink_do_rev	! do reverse
	bra	bl_quit

blink_do_rev:
	move.w	#BLINK_COUNT,(a0)
	lea	blink_stat,a0
	move.w	(a0),d0
	bne	bl_curoff	! not 0 ( CURSOR ON ) then next OFF
	add.w	#1,(a0)
	bsr	cursor_reverse
	bra	bl_quit
bl_curoff:
	clr.w	(a0)
	bsr	cursor_reverse

bl_quit:
	movem.l	(sp)+,d0/a0
	move.w	(sp)+,sr	! enable interrupt
	rts

!
!	cursor control
!

cursor_on:
	move.l	a0,-(sp)
	lea	blink_stat,a0
	tst.w	(a0)
	bne	cur_on_quit
	add.w	#1,(a0)
	bsr	cursor_reverse
cur_on_quit:
	move.l	(sp)+,a0
	rts

cursor_off:
	move.l	a0,-(sp)
	lea	blink_stat,a0
	tst.w	(a0)
	beq	cur_off_quit
	clr.w	(a0)
	bsr	cursor_reverse
cur_off_quit:
	move.l	(sp)+,a0
	rts

!
!	cursor reverse
!

cursor_reverse:
	movem.l	d0-d1/a0-a1,-(sp)
	move.w	vplane,d0	! plane check
	bne	cursor_p1
	move.l	#TVRAM0,a0
	lea	plane0,a1	! plane 0
	bra	cursor_next
cursor_p1:
	move.l	#TVRAM1,a0
	lea	plane1,a1	! plane 1
cursor_next:
	move.w	XPOS(a1),d0
	move.w	YPOS(a1),d1

!	mulu	#0x80*16,d1	! y offset
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a0
	add.w	d0,a0		! now a1 has addr. of TVRAM to exor

	move.l	#ALLPLANE,d1	! for all planes
	bsr	select_plane	! select planes
				! Then, I can access all planes by access tvram.

	not.b	(a0)		! line 0
	not.b	0x0080(a0)	! line 1
	not.b	0x0100(a0)	! line 2
	not.b	0x0180(a0)	! line 3
	not.b	0x0200(a0)	! line 4
	not.b	0x0280(a0)	! line 5
	not.b	0x0300(a0)	! line 6
	not.b	0x0380(a0)	! line 7
	not.b	0x0400(a0)	! line 8
	not.b	0x0480(a0)	! line 9
	not.b	0x0500(a0)	! line 10
	not.b	0x0580(a0)	! line 11
	not.b	0x0600(a0)	! line 12
	not.b	0x0680(a0)	! line 13
	not.b	0x0700(a0)	! line 14
	not.b	0x0780(a0)	! line 15
	movem.l	(sp)+,d0-d1/a0-a1
	rts

!
! sub-routine: select pnales to put chars.
!
!    argument
!	d1.w:	plane combination:
!
!		p.c  | plane  1  :  0  ( relatively )
!             -------+----------------------------------
!                 0  |        x     x
!                 1  |        x     o
!                 2  |        o     x
!                 3  |        o     o
!
!    After you call this routine, access the tvram ( top address of Text VRAM )
!  to put characters in any plane.
!

select_plane:
	lsl.w	#4,d1
	and.w	#0x00f0,d1
	or.w	#0x0100,d1
	move.w	d1,CRTCR21
	rts

!
! Function:	Change Visible Screen
!
!    argument:	NONE
!

_change_screen:
	clr.w	cursor_stat
	move.l	a0,-(sp)
	bsr	cursor_off
	lea	vplane,a0
	tst.w	(a0)
	bne	pl1_to_pl0
pl0_to_pl1:
	add.w	#1,(a0)
	move.w	#512,CRTCR11
	bra	change_screen_quit
pl1_to_pl0:
	clr.w	(a0)
	clr.w	CRTCR11
change_screen_quit:
	move.l	(sp)+,a0
	move.w	#1,cursor_stat
	rts

!
! Function:	Move Cursor Position
!
!    argument:
!	 (sp).w	plane no. ( 0 or 1 )
!	2(sp).w	cursor X point
!	4(sp).w	cursor Y point
!

_move_to:
	clr.w	cursor_stat
	movem.l	d0-d3/a0,-(sp)
	move.w	24(sp),d0
	move.w	26(sp),d1
	move.w	28(sp),d2

	lea	vplane,a0
	move.w	(a0),d3
	cmp.w	d0,d3		! if move cursor on visible screen, off cursor
	bne	move_next	!
	bsr	cursor_off	!
move_next:
	tst.w	d0		! plane check
	bne	move_pl1
move_pl0:
	lea	plane0,a0
	move.w	d0,(a0)+
	move.w	d1,(a0)
	bra	move_quit
move_pl1:
	lea	plane1,a0
	move.w	d0,(a0)+
	move.w	d1,(a0)
move_quit:
	movem.l	(sp)+,d0-d3/a0
	move.w	#1,cursor_stat
	rts

!
! Function:	Character Put
!
!    argument:
!	 (sp).w	plane no. to put ( 0 or 1 ) ---> d1
!	2(sp).w	character code to put out   ---> d0
!	
!    return:
!	d0.l	high 16bits : text X point after cursor movement
!	d0.l	 low 16bits : text Y point after cursor movement
!

_put_c:
	move.l	#0,d0
	move.w	d0,cursor_stat	!clr.w	cursor_stat

	movem.l	d1-d7/a0-a6,-(sp)

	move.b	63(sp),d0	! char code
	move.w	60(sp),d7	! plane no.
	!tst.w	d7		! TVRAM select
	bne	putc_pl1
putc_pl0:
	move.l	#TVRAM0,a5
	lea	plane0,a6
	bra	sel_plane_next
putc_pl1:
	move.l	#TVRAM1,a5
	lea	plane1,a6
sel_plane_next:
	move.w	vplane,d2
	cmp.w	d7,d2
	bne	putc_next
	bsr	cursor_off
putc_next:
	tst.w	ESC_STATE(a6)
	bne	parse_escape
	tst.b	KFLAG(a6)
	bne	kanput
	bsr	iskanji		! check kanji 1st char
	tst.w	d1
	beq	hanput		! not kanji 1st char
	move.b	#1,KFLAG(a6)
	move.b	d0,KFLAG+1(a6)
	bra	putc_quit

hanput:
	move.b	d0,d1
	and.b	#0xe0,d1
	bne	normal		! normal char
	move.l	d0,d2
	lsl.l	#2,d2		! x 4
	lea	jumptable,a0
	add.l	d2,a0
	jmp	(a0)

putc_curon_quit:
	bsr	cursor_on
putc_quit:
	move.l	XPOS(a6),d0
	movem.l	(sp)+,d1-d7/a0-a6
	move.w	#1,cursor_stat
	rts

jumptable:
	bra	dummy		! 00
	bra	dummy		! 01
	bra	dummy		! 02
	bra	dummy		! 03
	bra	dummy		! 04
	bra	dummy		! 05
	bra	dummy		! 06
	bra	BEL		! 07   BEL  Sound bell
	bra	BS		! 08   BS   Cursor backward
	bra	HT		! 09   HT   Skip to next tab stop
	bra	LF		! 0a   LF   Cursor down
	bra	VT		! 0b   VT   Cursor up
	bra	FF		! 0c   FF   Cursor forward
	bra	CR		! 0d   CR   Cursor to left margin
	bra	dummy		! 0e   
	bra	dummy		! 0f
	bra	dummy		! 10
	bra	dummy		! 11
	bra	dummy		! 12
	bra	dummy		! 13
	bra	dummy		! 14
	bra	dummy		! 15
	bra	dummy		! 16
	bra	dummy		! 17
	bra	dummy		! 18
	bra	dummy		! 19
	bra	SUB		! 1a   SUB  Clear screen
	bra	ESC		! 1b   ESC  Introduce on ESC sequence
	bra	dummy		! 1c
	bra	dummy		! 1d
	bra	RS		! 1e   RS   Cursor HOME
	bra	dummy		! 1f

!
!	Normal Char Put ( HANKAKU ASCII )
!

normal:
	cmp.b	#'\\',d0
	bne	normal1
	move.b	#0x80,d0
	bra	normal2

normal1:
	cmp.b	#'~',d0
	bne	normal2
	move.b	#0x81,d0
	
normal2:
	lsl.l	#4,d0			! x 16
	move.l	#CGROM8x16,a0
	add.l	d0,a0			! a0 has CGROM addr. for char. put

	move.w	XPOS(a6),d0
	move.w	YPOS(a6),d1

	move.l	a5,a1
!	mulu	#0x80*16,d1		! y offset
	ext.l	d1
	swap	d1
	lsr.l	#5,d1			! 128 x 16
	add.l	d1,a1
	add.w	d0,a1			! now a1 has addr. of TVRAM to put char.
	move.l	a1,a2

normal_chr_erase:
	cmp.w	#ALLPLANE,MASK(a6)
	beq	normal_chr_write

	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.l	#0,d1
	move.b	d1,(a2)			! line 0
	move.b	d1,0x0080(a2)		! line 1
	move.b	d1,0x0100(a2)		! line 2
	move.b	d1,0x0180(a2)		! line 3
	move.b	d1,0x0200(a2)		! line 4
	move.b	d1,0x0280(a2)		! line 5
	move.b	d1,0x0300(a2)		! line 6
	move.b	d1,0x0380(a2)		! line 7
	move.b	d1,0x0400(a2)		! line 8
	move.b	d1,0x0480(a2)		! line 9
	move.b	d1,0x0500(a2)		! line 10
	move.b	d1,0x0580(a2)		! line 11
	move.b	d1,0x0600(a2)		! line 12
	move.b	d1,0x0680(a2)		! line 13
	move.b	d1,0x0700(a2)		! line 14
	move.b	d1,0x0780(a2)		! line 15

normal_chr_write:
	move.w	MASK(a6),d1
	bsr	select_plane

	tst.w	REVERSE(a6)
	bne	normal_chr_reverse

	move.b	(a0)+,(a1)		! line 0
	move.b	(a0)+,0x0080(a1)	! line 1
	move.b	(a0)+,0x0100(a1)	! line 2
	move.b	(a0)+,0x0180(a1)	! line 3
	move.b	(a0)+,0x0200(a1)	! line 4
	move.b	(a0)+,0x0280(a1)	! line 5
	move.b	(a0)+,0x0300(a1)	! line 6
	move.b	(a0)+,0x0380(a1)	! line 7
	move.b	(a0)+,0x0400(a1)	! line 8
	move.b	(a0)+,0x0480(a1)	! line 9
	move.b	(a0)+,0x0500(a1)	! line 10
	move.b	(a0)+,0x0580(a1)	! line 11
	move.b	(a0)+,0x0600(a1)	! line 12
	move.b	(a0)+,0x0680(a1)	! line 13
	move.b	(a0)+,0x0700(a1)	! line 14
	move.b	(a0)+,0x0780(a1)	! line 15

	add.w	#1,d0			! x increment
	cmp.w	#MAXX,d0
	bhi	CRLF
	move.w	d0,XPOS(a6)
	bra	putc_quit

normal_chr_reverse:
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,(a1)			! line 0
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0080(a1)		! line 1
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0100(a1)		! line 2
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0180(a1)		! line 3
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0200(a1)		! line 4
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0280(a1)		! line 5
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0300(a1)		! line 6
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0380(a1)		! line 7
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0400(a1)		! line 8
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0480(a1)		! line 9
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0500(a1)		! line 10
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0580(a1)		! line 11
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0600(a1)		! line 12
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0680(a1)		! line 13
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0700(a1)		! line 14
	move.b	(a0)+,d1
	not.b	d1
	move.b	d1,0x0780(a1)		! line 15

	add.w	#1,d0			! x increment
	cmp.w	#MAXX,d0
	bhi	CRLF
	move.w	d0,XPOS(a6)
	bra	putc_quit

!
!	Kanji Char Put ( ZENKAKU CHAR )
!

kanput:
	move.w	KFLAG(a6),d1
	lsl.w	#8,d1
	move.b	d0,d1			! character code
	move.l	#8,d2			! 16 x 16 char
	move.l	_FNTADR*4+0x0400.w,a0
	jsr	(a0)
	move.l	d0,a4			! a4 has CGROM addr. for char to put
	clr.w	KFLAG(a6)

	move.w	XPOS(a6),d0
	move.w	YPOS(a6),d1
	cmp.w	#MAXX,d0
	bne	kanput2

kanput_clr_lastchr:
	move.l	a5,a1
!	mulu	#0x80*16,d1		! y offset
	ext.l	d1
	swap	d1
	lsr.l	#5,d1			! 128 x 16
	add.l	d1,a1
	add.w	d0,a1			! now a1 has addr. of TVRAM to put char.

	move.l	#0,d5
	move.b	d5,(a1)			! line 0
	move.b	d5,0x0080(a1)		! line 1
	move.b	d5,0x0100(a1)		! line 2
	move.b	d5,0x0180(a1)		! line 3
	move.b	d5,0x0200(a1)		! line 4
	move.b	d5,0x0280(a1)		! line 5
	move.b	d5,0x0300(a1)		! line 6
	move.b	d5,0x0380(a1)		! line 7
	move.b	d5,0x0400(a1)		! line 8
	move.b	d5,0x0480(a1)		! line 9
	move.b	d5,0x0500(a1)		! line 10
	move.b	d5,0x0580(a1)		! line 11
	move.b	d5,0x0600(a1)		! line 12
	move.b	d5,0x0680(a1)		! line 13
	move.b	d5,0x0700(a1)		! line 14
	move.b	d5,0x0780(a1)		! line 15

	move.w	d5,XPOS(a6)		! clr.w XPOS(a6)
	move.w	YPOS(a6),d0
	add.w	#1,d0
	cmp.w	#MAXY,d0
	bhi	kanput_scrollup
	move.w	d0,YPOS(a6)
	bra	kanput1
kanput_scrollup:
	bsr	scrollup

kanput1:
	move.w	XPOS(a6),d0
	move.w	YPOS(a6),d1
kanput2:
	move.l	a5,a1
!	mulu	#0x80*16,d1		! y offset
	ext.l	d1
	swap	d1
	lsr.l	#5,d1			! 128 x 16
	add.l	d1,a1
	add.w	d0,a1			! now a1 has addr. of TVRAM to put char.
	move.l	a1,a2

	move.l	a1,d1
	and.w	#0x0001,d1
	bne	kanput_odd

kanput_even:
	cmp.w	#ALLPLANE,MASK(a6)
	beq	kanput_even_chr

	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.l	#0,d5
	move.w	d5,(a2)			! line 0
	move.w	d5,0x0080(a2)		! line 1
	move.w	d5,0x0100(a2)		! line 2
	move.w	d5,0x0180(a2)		! line 3
	move.w	d5,0x0200(a2)		! line 4
	move.w	d5,0x0280(a2)		! line 5
	move.w	d5,0x0300(a2)		! line 6
	move.w	d5,0x0380(a2)		! line 7
	move.w	d5,0x0400(a2)		! line 8
	move.w	d5,0x0480(a2)		! line 9
	move.w	d5,0x0500(a2)		! line 10
	move.w	d5,0x0580(a2)		! line 11
	move.w	d5,0x0600(a2)		! line 12
	move.w	d5,0x0680(a2)		! line 13
	move.w	d5,0x0700(a2)		! line 14
	move.w	d5,0x0780(a2)		! line 15

kanput_even_chr:
	move.w	MASK(a6),d1
	bsr	select_plane

	tst.w	REVERSE(a6)
	bne	kanput_even_reverse

kanput_even_normal:
	move.w	(a4)+,(a1)		! line 0
	move.w	(a4)+,0x0080(a1)	! line 1
	move.w	(a4)+,0x0100(a1)	! line 2
	move.w	(a4)+,0x0180(a1)	! line 3
	move.w	(a4)+,0x0200(a1)	! line 4
	move.w	(a4)+,0x0280(a1)	! line 5
	move.w	(a4)+,0x0300(a1)	! line 6
	move.w	(a4)+,0x0380(a1)	! line 7
	move.w	(a4)+,0x0400(a1)	! line 8
	move.w	(a4)+,0x0480(a1)	! line 9
	move.w	(a4)+,0x0500(a1)	! line 10
	move.w	(a4)+,0x0580(a1)	! line 11
	move.w	(a4)+,0x0600(a1)	! line 12
	move.w	(a4)+,0x0680(a1)	! line 13
	move.w	(a4)+,0x0700(a1)	! line 14
	move.w	(a4)+,0x0780(a1)	! line 15

	add.w	#2,d0			! x increment
	cmp.w	#MAXX,d0
	bhi	CRLF
	move.w	d0,XPOS(a6)
	bra	putc_quit

kanput_even_reverse:
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,(a1)			! line 0
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0080(a1)		! line 1
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0100(a1)		! line 2
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0180(a1)		! line 3
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0200(a1)		! line 4
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0280(a1)		! line 5
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0300(a1)		! line 6
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0380(a1)		! line 7
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0400(a1)		! line 8
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0480(a1)		! line 9
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0500(a1)		! line 10
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0580(a1)		! line 11
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0600(a1)		! line 12
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0680(a1)		! line 13
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0700(a1)		! line 14
	move.w	(a4)+,d5
	not.w	d5
	move.w	d5,0x0780(a1)		! line 15

	add.w	#2,d0			! x increment
	cmp.w	#MAXX,d0
	bhi	CRLF
	move.w	d0,XPOS(a6)
	bra	putc_quit

kanput_odd:
	cmp.w	#ALLPLANE,MASK(a6)
	beq	kanput_odd_chr

	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.l	#0,d5
	move.b	d5,(a2)
	move.b	d5,0x0001(a2)		! line 0
	move.b	d5,0x0080(a2)
	move.b	d5,0x0081(a2)		! line 1
	move.b	d5,0x0100(a2)
	move.b	d5,0x0101(a2)		! line 2
	move.b	d5,0x0180(a2)
	move.b	d5,0x0181(a2)		! line 3
	move.b	d5,0x0200(a2)
	move.b	d5,0x0201(a2)		! line 4
	move.b	d5,0x0280(a2)
	move.b	d5,0x0281(a2)		! line 5
	move.b	d5,0x0300(a2)
	move.b	d5,0x0301(a2)		! line 6
	move.b	d5,0x0380(a2)
	move.b	d5,0x0381(a2)		! line 7
	move.b	d5,0x0400(a2)
	move.b	d5,0x0401(a2)		! line 8
	move.b	d5,0x0480(a2)
	move.b	d5,0x0481(a2)		! line 9
	move.b	d5,0x0500(a2)
	move.b	d5,0x0501(a2)		! line 10
	move.b	d5,0x0580(a2)
	move.b	d5,0x0581(a2)		! line 11
	move.b	d5,0x0600(a2)
	move.b	d5,0x0601(a2)		! line 12
	move.b	d5,0x0680(a2)
	move.b	d5,0x0681(a2)		! line 13
	move.b	d5,0x0700(a2)
	move.b	d5,0x0701(a2)		! line 14
	move.b	d5,0x0780(a2)
	move.b	d5,0x0781(a2)		! line 15

kanput_odd_chr:
	move.w	MASK(a6),d1
	bsr	select_plane

	tst.w	REVERSE(a6)
	bne	kanput_odd_reverse

kanput_odd_normal:
	move.b	(a4)+,(a1)		! line 0
	move.b	(a4)+,0x0001(a1)
	move.b	(a4)+,0x0080(a1)	! line 1
	move.b	(a4)+,0x0081(a1)
	move.b	(a4)+,0x0100(a1)	! line 2
	move.b	(a4)+,0x0101(a1)
	move.b	(a4)+,0x0180(a1)	! line 3
	move.b	(a4)+,0x0181(a1)
	move.b	(a4)+,0x0200(a1)	! line 4
	move.b	(a4)+,0x0201(a1)
	move.b	(a4)+,0x0280(a1)	! line 5
	move.b	(a4)+,0x0281(a1)
	move.b	(a4)+,0x0300(a1)	! line 6
	move.b	(a4)+,0x0301(a1)
	move.b	(a4)+,0x0380(a1)	! line 7
	move.b	(a4)+,0x0381(a1)
	move.b	(a4)+,0x0400(a1)	! line 8
	move.b	(a4)+,0x0401(a1)
	move.b	(a4)+,0x0480(a1)	! line 9
	move.b	(a4)+,0x0481(a1)
	move.b	(a4)+,0x0500(a1)	! line 10
	move.b	(a4)+,0x0501(a1)
	move.b	(a4)+,0x0580(a1)	! line 11
	move.b	(a4)+,0x0581(a1)
	move.b	(a4)+,0x0600(a1)	! line 12
	move.b	(a4)+,0x0601(a1)
	move.b	(a4)+,0x0680(a1)	! line 13
	move.b	(a4)+,0x0681(a1)
	move.b	(a4)+,0x0700(a1)	! line 14
	move.b	(a4)+,0x0701(a1)
	move.b	(a4)+,0x0780(a1)	! line 15
	move.b	(a4)+,0x0781(a1)

	add.w	#2,d0			! x increment
	cmp.w	#MAXX,d0
	bhi	CRLF
	move.w	d0,XPOS(a6)
	bra	putc_quit

kanput_odd_reverse:
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,(a1)
	lsr.w	#4,d1
	move.b	d1,0x0001(a1)		! line 0
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0080(a1)
	lsr.w	#4,d1
	move.b	d1,0x0081(a1)		! line 1
	add.w	#0x7f,a1
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0100(a1)
	lsr.w	#4,d1
	move.b	d1,0x0101(a1)		! line 2
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0180(a1)
	lsr.w	#4,d1
	move.b	d1,0x0181(a1)		! line 3
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0200(a1)
	lsr.w	#4,d1
	move.b	d1,0x0201(a1)		! line 4
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0280(a1)
	lsr.w	#4,d1
	move.b	d1,0x0281(a1)		! line 5
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0300(a1)
	lsr.w	#4,d1
	move.b	d1,0x0301(a1)		! line 6
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0380(a1)
	lsr.w	#4,d1
	move.b	d1,0x0381(a1)		! line 7
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0400(a1)
	lsr.w	#4,d1
	move.b	d1,0x0401(a1)		! line 8
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0480(a1)
	lsr.w	#4,d1
	move.b	d1,0x0481(a1)		! line 9
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0500(a1)
	lsr.w	#4,d1
	move.b	d1,0x0501(a1)		! line 10
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0580(a1)
	lsr.w	#4,d1
	move.b	d1,0x0581(a1)		! line 11
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0600(a1)
	lsr.w	#4,d1
	move.b	d1,0x0601(a1)		! line 12
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0680(a1)
	lsr.w	#4,d1
	move.b	d1,0x0681(a1)		! line 13
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0700(a1)
	lsr.w	#4,d1
	move.b	d1,0x0701(a1)		! line 14
	move.w	(a4)+,d1
	not.w	d1
	move.b	d1,0x0780(a1)
	lsr.w	#4,d1
	move.b	d1,0x0781(a1)		! line 15

	add.w	#2,d0			! x increment
	cmp.w	#MAXX,d0
	bhi	CRLF
	move.w	d0,XPOS(a6)
	bra	putc_quit

!
! Function:	Test for kanji header
!
!    argument:
!	d0.l	character for test
!
!    return:
!	d1.w	result:
!			0	not kanji
!			1	so kanji

iskanji:
	clr.w	d1
	cmp.b	#0x81,d0
	bcs	nokanji
	cmp.b	#0x9f,d0
	bls	okkanji
	cmp.b	#0xe0,d0
	bcs	nokanji
	cmp.b	#0xfc,d0
	bhi	nokanji
okkanji:
	move.w	#1,d1
nokanji:
	rts

!
!  Local Function: Scrollup Text Screen
!
!  This function use d0-d5/a0 and need a5 to set TVRAM top address
!  

scrollup:
	!clr.l	d0
	!clr.l	d1
	!clr.l	d2
	!clr.l	d3
	move.w	CRTCR21,d4	! save CRTC reg.
	move.w	CRTCR22,d5

	tst.w	d7		! check plane no. (T-T)
	bne	scrollup_pl1
scrollup_pl0:
	move.l	#0,d0		! moveq.l #0,d0
	move.l	#4,d1		! moveq.l #4,d1
	move.w	#128,d2
	bra	scrollup_loop

scrollup_pl1:
	move.w	#128,d0
	move.w	#128+4,d1
	move.w	#256,d2

scrollup_loop:
	move.b	d1,d3
	lsl.w	#8,d3
	move.b	d0,d3

	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	move.w	#ALLPLANE,CRTCR21
	move.w	d3,CRTCR22
	or.w	#0x0008,CRTCCTRL	! start raster copy
	bsr	wait_h_sync_minus
	add.w	#1,d0
	add.w	#1,d1
	cmp.w	d2,d1
	bne	scrollup_loop
	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	bsr	wait_h_sync_minus
	move.w	d4,CRTCR21		! restore CRTC reg.
	move.w	d5,CRTCR22

	move.l	a5,a0
	add.l	#0x80*16*32,a0
	movem.l	a1-a3,-(sp)
	move.l	#0,d1			! moveq.l #0,d1
	move.l	d1,d2
	move.l	d1,d3
	move.l	d1,d4
	move.l	d1,d5
	move.l	d1,a1
	move.l	d1,a2
	move.l	d1,a3
	move.w	#4-1,d0
scrollup_clear:
	movem.l	d1-d5/a1-a3,-(a0)	! clear 1 line
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)

	movem.l	d1-d5/a1-a3,-(a0)	! clear 1 line
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)

	movem.l	d1-d5/a1-a3,-(a0)	! clear 1 line
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)

	movem.l	d1-d5/a1-a3,-(a0)	! clear 1 line
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)
	movem.l	d1-d5/a1-a3,-(a0)

	dbra	d0,scrollup_clear
	movem.l	(sp)+,a1-a3

	rts

!
!	Wait H-SYNC ( for raster scroll )
!

wait_h_sync_plus:
	tst.b	MFPGPIP
	bpl	wait_h_sync_plus
	rts

wait_h_sync_minus:
	tst.b	MFPGPIP
	bmi	wait_h_sync_minus
	rts
	
wait_h_sync:
	move.l	d0,-(sp)
wait_h_sync1:
	move.b	MFPGPIP,d0
	and.b	#0x80,d0
	beq	wait_h_sync1
wait_h_sync2:
	move.b	MFPGPIP,d0
	and.b	#0x80,d0
	cmp.b	#0x80,d0
	beq	wait_h_sync2
	move.l	(sp)+,d0
	rts

!
!	Here are implementation of control chars
!

!**** FF: cursor X = X + 1 *****

FF:
	move.w	XPOS(a6),d0
	add.w	#1,d0
	cmp.w	#MAXX,d0
	bhi	x_over
	move.w	d0,XPOS(a6)
	bra	putc_quit
x_over:
	clr.w	XPOS(a6)
	bra	LF

!**** BS: cursor Y = Y - 1 *****

BS:
	sub.w	#1,XPOS(a6)
	bcc	putc_quit
x_minus:
	tst.w	YPOS(a6)		! if y == 0 then x = 0
	bne	next_bs2
	move.w	#0,XPOS(a6)
	bra	putc_curon_quit
next_bs2:
	move.w	#MAXX,XPOS(a6)
	sub.w	#1,YPOS(a6)
	bra	putc_curon_quit

!**** HT: hard tab *****

HT:
	move.w	XPOS(a6),d1
	lsr.w	#3,d1			! d1 / 8
	lsl.w	#3,d1
	add.w	#8,d1			! tab space = 8
	cmp.w	#MAXX,d1
	bhi	crlf_ht
	move.w	d1,XPOS(a6)
	bra	putc_curon_quit
crlf_ht:
	clr.w	XPOS(a6)		! bsr	CR
	bra	LF
	bra	putc_curon_quit

!**** CR *****

CR:
	clr.w	XPOS(a6)
	bra	putc_curon_quit

!**** LF: cursor Y = Y + 1 *****

CRLF:
	clr.w	XPOS(a6)
LF:
	move.w	YPOS(a6),d0
	add.w	#1,d0
	cmp.w	#MAXY,d0
	bhi	LF_scrollup
	move.w	d0,YPOS(a6)
	bra	putc_curon_quit

LF_scrollup:
	bsr	scrollup
	bra	putc_curon_quit

!**** VT: cursor Y = Y - 1 *****

VT:
	move.w	YPOS(a6),d0
	sub.w	#1,d0
	bcs	y_minus
	move.w	d0,YPOS(a6)
	bra	putc_curon_quit
y_minus:
	clr.w	YPOS(a6)
	bra	putc_curon_quit

!**** SUB: clear screen *****

SUB:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	! clear 1st raster of the screen
	lea	512(a5),a5
	movem.l	d6-d7,-(sp)
	move.l	#0,d0
	move.l	d0,d1
	move.l	d0,d2
	move.l	d0,d3
	move.l	d0,d4
	move.l	d0,d5
	move.l	d0,d6
	move.l	d0,d7

	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)

	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)

	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)

	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)
	movem.l	d0-d7,-(a5)

	movem.l	(sp)+,d6-d7
!cls_ras:
	move.w	CRTCR21,d4	! save CRTC reg.
	move.w	CRTCR22,d5

	tst.w	d7		! check plane no. (T-T)
	bne	cls_ras_pl1
!cls_ras_pl0:
	move.l	#1,d0		! to
	move.l	#0,d1		! from
	move.w	#128,d2
	bra	cls_ras_loop

cls_ras_pl1:
	move.w	#128+1,d0	! to
	move.w	#128,d1		! from
	move.w	#256,d2

cls_ras_loop:
	move.b	d1,d3
	lsl.w	#8,d3
	move.b	d0,d3

	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	move.w	#ALLPLANE,CRTCR21
	move.w	d3,CRTCR22
	or.w	#0x0008,CRTCCTRL	! start raster copy
	bsr	wait_h_sync_minus
	add.w	#1,d0
	cmp.w	d2,d0
	bne	cls_ras_loop
	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	bsr	wait_h_sync_minus
	move.w	d4,CRTCR21		! restore CRTC reg.
	move.w	d5,CRTCR22

!***** CAUTION: go through *****

!***** RS: cursor to home  *****

RS:
	clr.l	XPOS(a6)
	bra	putc_curon_quit

!**** ESC *****

ESC:
	move.w	#1,ESC_STATE(a6)
	clr.w	ESC_PARA_NUM(a6)
	clr.w	ESC_PARAMETER(a6)
	bra	putc_quit

!**** NOT IMPLEMENTED *****

BEL:
dummy:
	bra	putc_quit

!**** ESC M ****
EM:
	move.w	YPOS(a6),d0
	sub.w	#1,d0
	bcs	EM_scroll_down
	move.w	d0,YPOS(a6)
	bra	putc_curon_quit
EM_scroll_down:
	clr.w	YPOS(a6)
	move.w	#1,d6
	move.w	d6,d1
	move.w	XPOS(a6),-(sp)
	bsr	insert_scroll
	move.w	(sp)+,XPOS(a6)
	bra	putc_curon_quit

!
!	parse escape sequence
!

parse_escape:
	move.w	ESC_STATE(a6),d2
	cmp.w	#1,d2
	beq	level1
	cmp.w	#2,d2
	beq	level2
	clr.w	ESC_STATE(a6)
	bra	putc_quit
level1:
	cmp.b	#'[',d0
	beq	ctrl_seq_start
	clr.w	ESC_STATE(a6)
	cmp.b	#'*',d0
	beq	SUB
	cmp.b	#'D',d0
	beq	LF
	cmp.b	#'E',d0
	beq	CRLF
	cmp.b	#'M',d0
!	beq	VT
	beq	EM
!	clr.w	ESC_STATE(a6)
	bra	putc_quit

level2:
	cmp.b	#'0',d0
	bcs	l2_next
	cmp.b	#'9',d0
	bls	add_parameter
l2_next:
	cmp.b	#';',d0
	beq	next_parameter
	bra	do_escape

ctrl_seq_start:
	move.w	#2,ESC_STATE(a6)
	bra	putc_quit

add_parameter:
	move.l	#0,d2
	move.w	ESC_PARA_NUM(a6),d2
	add.w	d2,d2
	sub.b	#'0',d0

	move.w	ESC_PARAMETER(a6,d2.w),d1
	mulu	#10,d1
	add.w	d0,d1
	move.w	d1,ESC_PARAMETER(a6,d2.w)
	bra	putc_quit

next_parameter:
	move.w	ESC_PARA_NUM(a6),d1
	add.w	#1,d1
	move.w	d1,ESC_PARA_NUM(a6)

	add.w	d1,d1
	clr.w	ESC_PARAMETER(a6,d1.w)
	bra	putc_quit

do_escape:
	clr.w	ESC_STATE(a6)
	cmp.b	#'H',d0
	beq	move_cursor
	cmp.b	#'f',d0
	beq	move_cursor
	cmp.b	#'A',d0
	beq	up_cursor
	cmp.b	#'B',d0
	beq	down_cursor
	cmp.b	#'C',d0
	beq	forward_cursor
	cmp.b	#'D',d0
	beq	backward_cursor
	cmp.b	#'J',d0
	beq	clearJ
	cmp.b	#'K',d0
	beq	clearK
	cmp.b	#'M',d0
	beq	erase_line
	cmp.b	#'L',d0
	beq	insert_line
	cmp.b	#'m',d0
	beq	set_attribute
	bra	putc_quit

!
!	Here are implementation of control sequence
!

!**** Direct Cursor Addressing *****

move_cursor:
	move.w	ESC_PARAMETER(a6),d0	! Y pos
	beq	move_y_next
	sub.w	#1,d0
move_y_next:
	cmp.w	#MAXY,d0
	bls	y_le_max
	move.w	#MAXY,d0
y_le_max:
	move.w	d0,YPOS(a6)

	move.w	ESC_PARAMETER+2(a6),d0	! X pos
	beq	move_x_next
	sub.w	#1,d0
move_x_next:
	cmp.w	#MAXX,d0
	bls	x_le_max
	move.w	#MAXX,d0
x_le_max:
	move.w	d0,XPOS(a6)
	bra	putc_curon_quit

!**** Cursor Up *****

up_cursor:
	move.w	ESC_PARAMETER(a6),d0
	bne	up_cur_next1
	move.w	#1,d0
up_cur_next1:
	move.w	YPOS(a6),d1
	sub.w	d0,d1
	bcs	putc_curon_quit
	move.w	d1,YPOS(a6)
	bra	putc_curon_quit

!**** Cursor Down *****

down_cursor:
	move.w	ESC_PARAMETER(a6),d0
	bne	down_cur_next1
	move.w	#1,d0
down_cur_next1:
	move.w	YPOS(a6),d1
	add.w	d0,d1
	cmp.w	#MAXY,d1
	bhi	putc_curon_quit
	move.w	d1,YPOS(a6)
	bra	putc_curon_quit

!***** Cursor Forward *****

forward_cursor:
	move.w	ESC_PARAMETER(a6),d0
	bne	fwd_cur_next1
	move.w	#1,d0
fwd_cur_next1:
	move.w	XPOS(a6),d1
	add.w	d0,d1
	cmp.w	#MAXX,d1
	bhi	putc_curon_quit
	move.w	d1,XPOS(a6)
	bra	putc_curon_quit

!**** Cursor Backward *****

backward_cursor:
	move.w	ESC_PARAMETER(a6),d0
	bne	bwd_cur_next1
	move.w	#1,d0
bwd_cur_next1:
	move.w	XPOS(a6),d1
	sub.w	d0,d1
	bcs	putc_curon_quit
	move.w	d1,XPOS(a6)
	bra	putc_curon_quit

!***** Clear Screen *****

clearJ:
	move.w	ESC_PARAMETER(a6),d0
	beq	clearJ0
	cmp.w	#1,d0
	beq	clearJ1
	bra	SUB

clearJ0:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.w	XPOS(a6),d0		! get X and Y position
	move.w	YPOS(a6),d1

	move.l	a5,a1
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a1
	move.l	a1,a2
	add.l	#0x80*16,a2	! calc TVRAM addr. of cursor next line.
	add.w	d0,a1		! calc TVRAM addr. of cursor line.

clearJ0_curline:
	move.w	#0x7f,d2	! calc remaining char par line.
	sub.w	d0,d2

	move.w	#15,d4
clearJ0_curline1:
	move.l	a1,a3
	move.w	d2,d3
clearJ0_curline2:
	clr.b	(a3)+
	dbra	d3,clearJ0_curline2
	add.w	#0x80,a1
	dbra	d4,clearJ0_curline1

clearJ0_remain:
	move.w	YPOS(a6),d1
	lsl.w	#4,d1
	move.w	#512-1,d7	! calc remaining lines
	sub.w	d1,d7

	move.l	#0,d0
	move.l	d0,d1
	move.l	d0,d2
	move.l	d0,d3
	move.l	d0,d4
	move.l	d0,d5
	move.l	d0,d6
	move.l	d0,a0
clearJ0_remain1:
	lea	128(a2),a2
	move.l	a2,a1

	movem.l	d0-d6/a0,-(a1)	! clear 1 line
	movem.l	d0-d6/a0,-(a1)	! clear 1 line
	movem.l	d0-d6/a0,-(a1)	! clear 1 line
	movem.l	d0-d6/a0,-(a1)	! clear 1 line

	dbra	d7,clearJ0_remain1
	bra	putc_curon_quit

clearJ1:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.w	XPOS(a6),d0		! get X and Y position
	move.w	YPOS(a6),d1

	move.l	a5,a1
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a1		! calc TVRAM addr. of cursor line. ( X = 0 )

	move.l	a5,a2
	move.l	#0,d1
	move.l	#0,d2
	move.l	#0,d3
	move.l	#0,d4
clearJ1_to_curline:
	lea	128(a2),a2
	move.l	a2,a3
	movem.l	d1-d4,-(a3)	! clear 1 line
	movem.l	d1-d4,-(a3)
	movem.l	d1-d4,-(a3)
	movem.l	d1-d4,-(a3)
	movem.l	d1-d4,-(a3)
	movem.l	d1-d4,-(a3)
	movem.l	d1-d4,-(a3)
	movem.l	d1-d4,-(a3)
	cmp.l	a1,a2
	bcs	clearJ1_to_curline

clearJ1_curline:
	move.w	#0x7f,d2	! calc remaining char par line.
	sub.w	d0,d2

	move.w	#15,d4
clearJ1_curline1:
	move.l	a1,a3
	move.w	d2,d3
clearJ1_curline2:
	clr.b	(a3)+
	dbra	d3,clearJ1_curline2
	add.w	#0x80,a1
	dbra	d4,clearJ1_curline1
	bra	putc_curon_quit

!**** Clear Cursor Line *****

clearK:
	move.w	ESC_PARAMETER(a6),d0
	beq	clearK0
	cmp.w	#1,d0
	beq	clearK1

clearK2:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.w	YPOS(a6),d1

	move.l	a5,a1
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a1		! calc TVRAM addr. of cursor line.

	move.l	#0,d0
	move.l	d0,d1
	move.l	d0,d2
	move.l	d0,d3
	move.l	d0,d4
	move.l	d0,a3
	move.l	d0,a4
	move.l	d0,a5
	move.w	#16-1,d5
clearK2_1line:
	lea	128(a1),a1
	move.l	a1,a2
	movem.l	d0-d4/a3-a5,-(a2)	! clear 1 line
	movem.l	d0-d4/a3-a5,-(a2)
	movem.l	d0-d4/a3-a5,-(a2)
	movem.l	d0-d4/a3-a5,-(a2)
	dbra	d5,clearK2_1line
	bra	putc_curon_quit

clearK0:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.w	XPOS(a6),d0		! get X and Y position
	move.w	YPOS(a6),d1

	move.l	a5,a1
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a1
	add.w	d0,a1		! calc TVRAM addr of cursor pos

	move.w	#0x7f,d2	! calc remaining char par line.
	sub.w	d0,d2

	move.w	#15,d4
clearK0_1:
	move.l	a1,a3
	move.w	d2,d3
clearK0_2:
	clr.b	(a3)+
	dbra	d3,clearK0_2
	add.w	#0x80,a1
	dbra	d4,clearK0_1
	bra	putc_curon_quit

clearK1:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	move.w	XPOS(a6),d0	! get X and Y position
	move.w	YPOS(a6),d1

	move.l	a5,a1
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a1		! calc TVRAM addr. of cursor line. ( X = 0 )

	move.w	#0x7f,d2	! calc remaining char par line.
	sub.w	d0,d2

	move.w	#15,d4
clearK1_1:
	move.l	a1,a3
	move.w	d2,d3
clearK1_2:
	clr.b	(a3)+
	dbra	d3,clearK1_2
	add.w	#0x80,a1
	dbra	d4,clearK1_1
	bra	putc_curon_quit

!**** Delete Line *****

erase_line:
	move.w	ESC_PARAMETER(a6),d6
	bne	erase_line_next
	move.w	#1,d6
erase_line_next:
	move.w	YPOS(a6),d1
	add.w	d6,d1
	cmp.w	#MAXY,d1
	bls	erase_scroll
	move.w	#MAXY+1,d6
	sub.w	YPOS(a6),d6		! calc number of delete lines

erase_scroll:
	move.w	YPOS(a6),d0
	move.w	d0,d1
	add.w	d6,d1
	add.w	d0,d0			! x 4 ( for raster copy )
	add.w	d0,d0
	add.w	d1,d1			! x 4 ( for raster copy )
	add.w	d1,d1
	move.l	#0,d2
	move.w	CRTCR21,d3		! save CRTC reg.
	move.w	CRTCR22,d4

	tst.w	d7			! check plane number
	bne	erase_scroll_pl1
erase_scroll_pl0:
	cmp.w	#128,d1
	beq	erase_scroll_esc

	move.b	d1,d2
	lsl.w	#8,d2
	move.b	d0,d2			! calc CRTC data

	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	move.w	#ALLPLANE,CRTCR21
	move.w	d2,CRTCR22
	or.w	#0x0008,CRTCCTRL	! start raster copy
	bsr	wait_h_sync_minus
	add.w	#1,d0
	add.w	#1,d1
	bra	erase_scroll_pl0

erase_scroll_pl1:
	add.w	#128,d0
	add.w	#128,d1
erase_scroll_pl1_loop:
	cmp.w	#256,d1
	beq	erase_scroll_esc

	move.b	d1,d2
	lsl.w	#8,d2
	move.b	d0,d2			! calc CRTC data

	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	move.w	#ALLPLANE,CRTCR21
	move.w	d2,CRTCR22
	or.w	#0x0008,CRTCCTRL	! start raster copy
	bsr	wait_h_sync_minus
	add.w	#1,d0
	add.w	#1,d1
	bra	erase_scroll_pl1_loop

erase_scroll_esc:
	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	bsr	wait_h_sync_minus
	move.w	d3,CRTCR21
	move.w	d4,CRTCR22

erase_line_body:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	clr.w	XPOS(a6)	! clear cursor X position
	move.w	#MAXY+1,d1
	sub.w	d6,d1
	move.l	a5,a0
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a0		! calc TVRAM addr. of cursor line.

	move.w	d6,d0		! restore number of delete lines
	sub.w	#1,d0

	move.l	#0,d2
	move.l	d2,d3
	move.l	d2,d4
	move.l	d2,d5
	move.l	d2,d6
	move.l	d2,d7
	move.l	d2,a1
	move.l	d2,a2
erase_line_loop:
	move.w	#16-1,d1
erase_1line:
	lea	128(a0),a0
	move.l	a0,a3
	movem.l	d2-d7/a1-a2,-(a3)	! clear 1 line
	movem.l	d2-d7/a1-a2,-(a3)
	movem.l	d2-d7/a1-a2,-(a3)
	movem.l	d2-d7/a1-a2,-(a3)
	dbra	d1,erase_1line
	dbra	d0,erase_line_loop
	bra	putc_quit

!**** Insert Line *****

insert_line:
	move.w	ESC_PARAMETER(a6),d6
	bne	insert_line_next
	move.w	#1,d6
insert_line_next:
	move.w	YPOS(a6),d1
	add.w	d6,d1
	cmp.w	#MAXY,d1
	bhi	clearK2

	bsr	insert_scroll
	bra	putc_quit

insert_scroll:
	move.w	YPOS(a6),d5
	add.w	d5,d5
	add.w	d5,d5
	sub.w	#1,d5			! raster copy end

	move.w	#MAXY,d0
	move.w	d0,d1
	sub.w	d6,d1
	add.w	d0,d0
	add.w	d0,d0
	add.w	#3,d0			! x 4 + 3 ( for raster copy )
	add.w	d1,d1
	add.w	d1,d1
	add.w	#3,d1			! x 4 + 3 ( for raster copy )
	move.l	#0,d2
	move.w	CRTCR21,d3		! save CRTC reg.
	move.w	CRTCR22,d4

	tst.w	d7
	bne	insert_scroll_pl1
insert_scroll_pl0:
	cmp.w	d5,d1
	beq	insert_scroll_esc

	move.b	d1,d2
	lsl.w	#8,d2
	move.b	d0,d2

	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	move.w	#ALLPLANE,CRTCR21
	move.w	d2,CRTCR22
	or.w	#0x0008,CRTCCTRL	! start raster copy
	bsr	wait_h_sync_minus
	sub.w	#1,d0
	sub.w	#1,d1
	bra	insert_scroll_pl0

insert_scroll_pl1:
	add.w	#128,d0
	add.w	#128,d1
	add.w	#128,d5			! end of raster copy (kat)
insert_scroll_pl1_loop:
	cmp.w	d5,d1
	beq	insert_scroll_esc

	move.b	d1,d2
	lsl.w	#8,d2
	move.b	d0,d2			! calc CRTC data

	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	move.w	#ALLPLANE,CRTCR21
	move.w	d2,CRTCR22
	or.w	#0x0008,CRTCCTRL	! start raster copy
	bsr	wait_h_sync_minus
	sub.w	#1,d0
	sub.w	#1,d1
	bra	insert_scroll_pl1_loop

insert_scroll_esc:
	bsr	wait_h_sync_plus
	and.w	#0x0007,CRTCCTRL	! stop raster copy
	bsr	wait_h_sync_minus
	move.w	d3,CRTCR21
	move.w	d4,CRTCR22

insert_line_body:
	move.w	#ALLPLANE,d1
	bsr	select_plane

	clr.w	XPOS(a6)	! clear cursor X position
	move.w	YPOS(a6),d1
	move.l	a5,a0
!	mulu	#0x80*16,d1
	ext.l	d1
	swap	d1
	lsr.l	#5,d1		! 128 x 16
	add.l	d1,a0		! calc TVRAM addr. of cursor line.

	move.l	#0,d2
	move.l	d2,d3
	move.l	d2,d4
	move.l	d2,d5
	move.l	d2,a2
	move.l	d2,a3
	move.l	d2,a4
	move.l	d2,a5
	move.w	d6,d0		! restore number of delete lines
	sub.w	#1,d0
insert_line_loop:
	move.w	#16-1,d1
insert_1line:
	lea	128(a0),a0
	move.l	a0,a1
	movem.l	d2-d5/a2-a5,-(a1) ! clear 1 line
	movem.l	d2-d5/a2-a5,-(a1)
	movem.l	d2-d5/a2-a5,-(a1)
	movem.l	d2-d5/a2-a5,-(a1)
	dbra	d1,insert_1line
	dbra	d0,insert_line_loop
	rts

!**** Character Attribute *****

set_attribute:
	move.w	ESC_PARAMETER(a6),d0
	move.w	d0,d1
	lsr.w	#4,d1
	and.w	#0x000f,d0
	beq	default_attribute
	and.w	#0x0001,d1
	move.w	d0,MASK(a6)
	move.w	d1,REVERSE(a6)
	bra	putc_quit

default_attribute:
	move.w	#ALLPLANE,MASK(a6)
	clr.w	REVERSE(a6)
	bra	putc_quit
