#if (MACHINE == X68000)

/* This driver does not deal with multiple consoles and parameter passing
   through tp. Also the tty struct fields row and column are not maintained
*/

/*
 * The video driver for the Sharp X68000.
 */
#include "kernel.h"
#include <minix/com.h>
#include <sgtty.h>

#if 0
#include "staddr.h"
#include "stvideo.h"
#include "stsound.h"
#endif

#include "tty.h"

/* #define USE_BELL_PCM_C */

#ifndef USE_BELL_PCM_C
#define	PCM_SIZE	0x200
PRIVATE char	pcm_data_buf[PCM_SIZE];
#else
#include "bellpcm.c"
#endif /* USE_BELL_PCM_C */

/* Constants relating to the video RAM and fonts */
#define NCOL		96	/* characters on a row */
#define	PIX_LIN		768	/* pixels per video line */
#define	PIX_CHR		8	/* pixels in char (width) */
#define	M_LIN_SCR	512	/* video lines on mono screen */
#define	C_LIN_SCR	512	/* video lines on color screen */
#if 0
#define	VIDEORAM	32000	/* size of video ram */
#define NCOL		80	/* characters on a row */
#define	PIX_LIN		640	/* pixels per video line */
#define	PIX_CHR		8	/* pixels in char (width) */
#define	M_LIN_SCR	400	/* video lines on mono screen */
#define	C_LIN_SCR	200	/* video lines on color screen */
#define	M_BYT_LIN	80	/* bytes in mono video line */
#define	C_BYT_LIN	160	/* bytes in color video line */
#endif

PRIVATE struct vduinfo {
	char	*vram;		/* base of video ram */
	char	*curs;		/* cursor position in video RAM */
	int	linc;		/* video lines in char */
	int	nrow;		/* char rows on screen */
	int	bytr;		/* bytes in row of chars */
	char	mono;		/* 1 if mono, 0 if color */
	char	attr;		/* current attribute byte */
	int	ccol;		/* current char column */
	int	crow;		/* current char row */
	char	savattr;	/* saved attribute byte */
	int	savccol;	/* saved char column */
	int	savcrow;	/* saved char row */
	char	vbuf[20];	/* partial escape sequence */
	char	*next;		/* next char in vbuf[] */
} vduinfo;

PRIVATE int kanji_flag = FALSE;
PRIVATE message mess;			/* message buffer for in and out */

EXTERN void change_screen();
EXTERN void move_to();
EXTERN long put_c();
PRIVATE void vductrl();
PRIVATE void beep();
PUBLIC void beepinit();

/*===========================================================================*
 *				flush					     *
 *===========================================================================*/
EXTERN void flush(tp)
register struct tty_struct *tp;
{
  register char *rq;

  if (tp->tty_rwords == 0)
	return;
/*  vducur_off(tp->tty_line); */
  rq = (char *)tp->tty_ramqueue;
  do {
	if (tp->tty_inhibited == TRUE) {
/*		vducur_on(tp->tty_line); */
		return;
	}
	out_char(tp, *rq++);	/* write 1 byte to terminal */
	tp->tty_phys++;		/* advance physical data pointer */
	tp->tty_cum++;		/* number of characters printed */
  } while (--tp->tty_rwords != 0);
/*  vducur_on(tp->tty_line); */
  return;
}

/*===========================================================================*
 *				console					     *
 *===========================================================================*/
PRIVATE void console(tp)
register struct tty_struct *tp;	/* tells which terminal is to be used */
{
/* Copy as much data as possible to the output queue, then start I/O.  On
 * memory-mapped terminals, such as the IBM console, the I/O will also be
 * finished, and the counts updated.  Keep repeating until all I/O done.
 */

  int count = 0;
  char c;
#if (CHIP == M68000)
  char *charptr = (char *)tp->tty_phys;

/*  vducur_on(tp->tty_line); */
#else
  extern char get_byte();
  unsigned segment, offset, offset1;

  /* Loop over the user bytes one at a time, outputting each one. */
  segment = (tp->tty_phys >> 4) & WORD_MASK;
  offset = tp->tty_phys & OFF_MASK;
  offset1 = offset;
#endif

  while (tp->tty_outleft > 0 && tp->tty_inhibited == RUNNING) {
#if (CHIP == M68000)
	out_char(tp, *charptr++);	/* write 1 byte to terminal */
	count++;
#else
	c = get_byte(segment, offset);	/* fetch 1 byte from user space */
	out_char(tp, c);	/* write 1 byte to terminal */
	offset++;		/* advance one character in user buffer */
#endif
	tp->tty_outleft--;	/* decrement count */
  }
#if (CHIP == M68000)
/*  vducur_on(tp->tty_line); */
#endif
  flush(tp);			/* clear out the pending characters */

  /* Update terminal data structure. */
#if (CHIP != M68000)
  count = offset - offset1;	/* # characters printed */
#endif
  tp->tty_phys += count;	/* advance physical data pointer */
  tp->tty_cum += count;		/* number of characters printed */

  /* If all data has been copied to the terminal, send the reply. */
  if (tp->tty_outleft == 0) finish(tp, tp->tty_cum);
}

/*===========================================================================*
 *				out_char				     *
 *===========================================================================*/
/*
 * send character to VDU, collecting escape sequences
 */
PUBLIC void out_char(tp, c)
struct tty_struct *tp;
register char c;			/* character to be output */
{

#if 0
  if (c == 0x7F)
	return;
#endif

  if( ( c & 0xe0 ) == 0 && kanji_flag == FALSE ) vductrl(tp,c);
  else put_c( tp->tty_line, c );

  if( kanji_flag == TRUE ) kanji_flag = FALSE;
  else if( iskanji(c) )    kanji_flag = TRUE;
}

/*
 * control character
 */

PRIVATE void vductrl(tp, c)
struct tty_struct *tp;
char c;
{
  switch (c)
  {
  case 0x07: /* BEL */
	if( tp->tty_line == fg_console ) { beep(); return; }
  case 0x0a: /* LF */
	if( tp->tty_mode & CRMOD ) put_c( tp->tty_line, '\r' );
	/*** break through ***/
  case 0x08: /* BS */
  case 0x09: /* HT */
  case 0x0b: /* VT */
  case 0x0c: /* FF */
  case 0x0d: /* CR */
  case 0x1a: /* SUB */
  case 0x1b: /* ESC */
  case 0x1e: /* RS */
	put_c( tp->tty_line, c );
  default:
	return;
  }
}

/*===========================================================================*
 *				beepinit				     *
 *===========================================================================*/
PUBLIC void beepinit()
{
  unsigned int	i = 0;

#ifndef USE_BELL_PCM_C
  while (i < PCM_SIZE/2){
    pcm_data_buf[i++] = 0x11;
    pcm_data_buf[i++] = 0x11;
    pcm_data_buf[i++] = 0xff;
    pcm_data_buf[i++] = 0xff;
  }
  while(i < PCM_SIZE){
    pcm_data_buf[i++] = 0x11;
    pcm_data_buf[i++] = 0x11;
    pcm_data_buf[i++] = 0x00;
    pcm_data_buf[i++] = 0x00;
    pcm_data_buf[i++] = 0x00;
    pcm_data_buf[i++] = 0x00;
    pcm_data_buf[i++] = 0xff;
    pcm_data_buf[i++] = 0xff;
  }
#endif
}

/*===========================================================================*
 *				beep					     *
 *===========================================================================*/
PRIVATE void beep()
{
  mess.m_source = TTY;
  mess.m_type = ADPCM_WRITE;
  mess.DEVICE = 0;
  mess.PROC_NR = TTY;
  mess.COUNT = PCM_SIZE;
  mess.ADDRESS = pcm_data_buf;
  send(ADPCM, &mess);
}

/*===============================================*
 *               swap_screen                     *
 *===============================================*/
PUBLIC void swap_screen( old_console )
int old_console;
{
    int sr;
    struct tty_struct *fg, *old;

    if( fg_console == old_console ) return;
    change_screen();
}

/*===========================================================================*
 *				vduinit					     *
 *===========================================================================*/
PUBLIC void vduinit()
{
  int ttys;

  for( ttys = 0 ; ttys < NR_CONS ; ttys++ )
  {  
    struct tty_struct *tp = &tty_struct[ttys];

    tp->tty_devstart = (int (*)())console;
    move_to( tp->tty_line, 0, 0 );
	put_c( tp->tty_line, 0x1a );
  }
}
#endif
