/*
 * Sharp X68000 hardware:
 *  Hudson SCSI board
 *   NCR5380 - SCSI Controler
 */

/*
 * SCSI Controller Address & Feature
 */
#define	SDC	((struct sdc *)AD_HSC)

struct sdc {
	unsigned char	scb[128];	/* use only the odd bytes */
};

#define	sc_dat		scb[1]	/* current SCSI data register */
#define	sc_odt		scb[1]	/* output data register */
#define	sc_ini		scb[3]	/* initiator command register */
#define	sc_mod		scb[5]	/* mode register */
#define	sc_trg		scb[7]	/* target command register */
#define	sc_css		scb[9]	/* current SCSI bus status */
#define	sc_sel		scb[9]	/* select enable register */
#define	sc_bst		scb[11]	/* bus & status register */
#define	sc_sds		scb[11]	/* start DMA send */
#define	sc_idt		scb[13]	/* input data register */
#define	sc_sdt		scb[13]	/* start DMA target receive */
#define	sc_rpi		scb[15]	/* reset parity / interrupt */
#define	sc_sdi		scb[15]	/* start DMA initiator receive */
#define	sc_win		scb[33]	/* DMA window, image of sc_dat */
#define	sc_dsw		scb[65]	/* DIP switch on the board and DMA enable switch */

/*
 * SCSI Commands, to be written to low 5 bits of CDB
 */
#define	SC_TUR	0x00	/* Test Unit Ready */
#define	SC_RZR	0x01	/* ReZeRo; in fact a ReSeT + head to track 0 */
#define	SC_RQS	0x03	/* ReQuest Sense */
#define	SC_FMT	0x04	/* ForMaT UNIT */
#define	SC_RD	0x08	/* ReaD */
#define	SC_WR	0x0a	/* WRite */
#define	SC_SK	0x0b	/* SeeK */

/*
 * SCSI phase status bytes
 */
#define	PS_BUS	0x00	/* Bus Free Phase */
#define	PS_SEL	0x02	/* Selection Phase */
#define	PS_CMD	0x68	/* Command Phase */
#define	PS_DIN	0x64	/* Data In Phase */
#define	PS_DOT	0x60	/* Data Out Phase */
#define	PS_STT	0x6c	/* Status Phase */
#define	PS_MES	0x7c	/* Message Phase */

/*
 * bits in completion status byte:
 */
#define	HDC_CC	0x02	/* Check Condition */

/*
 * codes for byte 0 of status block returned by HD_RQS:
 */
#define	NOERROR		0x00
#define	NOINDEX		0x01
#define	NOSEEKCOMPLETE	0x02
#define	WRITEFAULT	0x03
#define	DRIVENOTREADY	0x04
#define	NOTRACKZERO	0x06
#define	IDECC		0x10
#define	BADDATA		0x11
#define	NOIDADDRMARK	0x12
#define	NODATAADDRMARK	0x13
#define	NORECORD	0x14
#define	BADSEEK		0x15
#define	DATANOECC	0x18
#define	DATAECC		0x19
#define	BADFORMAT	0x1c
#define	BADSELFTEST	0x1d
#define	BADOP		0x20
#define	BADBLOCKADDR	0x21
#define	BADFINALADDR	0x23
#define	BADARG		0x24
#define	BADDEVICE	0x25

/*
 * Partition information
 */
struct pi {
  long	pi_name[2];		/* OS name */
  long	pi_start;		/* first sector */
  long	pi_size;		/* total number of sectors */
};

#define	NPARTS		16

/*
 * Hard disk information (sector 4) of any hard disk drive
 */
struct hi {
  unsigned long	hi_start;	/* start sector */
  unsigned long	hi_size;	/* total number of sectors */
  unsigned long	hi_name[2];	/* OS name */
  unsigned char hi_act;		/* partition activity */
  unsigned char dummy1;
  unsigned short dummy2;
  unsigned long dummy3;
  char dummy4[8];
};

/* values of hi_act: */
#define	PI_BOOT		0x00
#define	PI_VALID	0x02
