/*
 * Sharp X68000 hardware:
 * TDC-510B - Hard Disk Drive and Controller
 */

/*
 * Hard Disk Controller Address
 */
#define	HDC	((struct hdc *)AD_HDC)

struct hdc {
	unsigned char	hdb[8];	/* use only the odd bytes */
};

#define	hd_dat		hdb[1]	/* HDC data register */
#define	hd_stt		hdb[3]	/* HDC status register */
#define	hd_sre		hdb[3]	/* HDC select reset register */
#define	hd_res		hdb[5]	/* HDC reset register */
#define	hd_sel		hdb[7]	/* HDC select register */

/*
 * Commands, to be written to low 5 bits of CDB
 */
#define	HD_TUR	0x00	/* Test Unit Ready */
#define	HD_RST	0x01	/* ReSTore; in fact a ReSeT + head to track 0 */
#define	HD_RQS	0x03	/* ReQuest Sense */
#define	HD_FDR	0x04	/* Format DRive */
#define	HD_RD	0x08	/* ReaD */
#define	HD_WR	0x0a	/* WRite */
#define	HD_SK	0x0b	/* SeeK */

/*
 * HDC phase status bytes
 */
#define	PS_BUS	0x00	/* Bus Free Phase */
#define	PS_SEL	0x02	/* Selection Phase */
#define	PS_CMD	0x0a	/* Command Phase */
#define	PS_DRD	0x06	/* Data Phase (read) */
#define	PS_DWT	0x02	/* Data Phase (write) */
#define	PS_STT	0x0e	/* Status Phase */
#define	PS_MES	0x1e	/* Message Phase */

#define PS_REQ_BIT  0x01

/*
 * bits in completion status byte:
 */
#define	HDC_CC	0x02	/* Check Condition */

/*
 * codes for byte 0 of status block returned by HD_RQS:
 */
#define	NOERROR		0x00
#define	NOINDEX		0x01
#define	NOSEEKCOMPLETE	0x02
#define	WRITEFAULT	0x03
#define	DRIVENOTREADY	0x04
#define	NOTRACKZERO	0x06
#define	IDECC		0x10
#define	BADDATA		0x11
#define	NOIDADDRMARK	0x12
#define	NODATAADDRMARK	0x13
#define	NORECORD	0x14
#define	BADSEEK		0x15
#define	DATANOECC	0x18
#define	DATAECC		0x19
#define	BADFORMAT	0x1c
#define	BADSELFTEST	0x1d
#define	BADOP		0x20
#define	BADBLOCKADDR	0x21
#define	BADFINALADDR	0x23
#define	BADARG		0x24
#define	BADDEVICE	0x25

/*
 * Partition information
 */
struct pi {
  long	pi_name[2];		/* OS name */
  long	pi_start;		/* first sector */
  long	pi_size;		/* total number of sectors */
};

/* values of pi_flag: */
#define	PI_INVALID	0x00
#define	PI_BOOT		0x80

#define	NPARTS		16

/*
 * Hard disk information (sector 4) of any hard disk drive
 */
struct hi {
  long	hi_name[2];		/* OS name */
  long	hi_start;		/* first sector */
  long	hi_size;		/* total number of sectors */
};
