/*
 * Sharp X68000 hardware:
 *  Sharp SCSI board
 *   MB89352 - SCSI Protocol Controler
 */

/*
 * SCSI Controller Address & Feature
 */
#define	INT_SCSI	((struct sdc *)AD_ISC)
#define	EXT_SCSI	((struct sdc *)AD_ESC)

struct sdc {
	unsigned char	scb[32];	/* use only the odd bytes */
};

#define	sc_bdid		scb[1]	/* Bus Device ID */
#define	sc_sctl		scb[3]	/* SPC Control */
#define	sc_scmd		scb[5]	/* SPC Command */
#define	sc_ints		scb[9]	/* Interrupt Sense */
#define	sc_psns		scb[11]	/* Phase Sense */
#define	sc_sdgc		scb[11]	/* SPC Diag Control */
#define	sc_ssts		scb[13]	/* SPC Status */
#define	sc_serr		scb[15]	/* SPC Error Status */
#define	sc_pctl		scb[17]	/* Phase Control */
#define	sc_mbc		scb[19]	/* Modified Byte Counter */
#define	sc_dreg		scb[21]	/* Data Register */
#define	sc_temp		scb[23]	/* Temporary Register */
#define	sc_tch		scb[25]	/* Transfer Counter High */
#define	sc_tcm		scb[27]	/* Transfer Counter Mid */
#define	sc_tcl		scb[29]	/* Transfer Counter Low */

/*
 * SCSI Commands
 */
#define	SC_TUR	0x00	/* Test Unit Ready */
#define	SC_RZR	0x01	/* ReZeRo; in fact a ReSeT + head to track 0 */
#define	SC_RQS	0x03	/* ReQuest Sense */
#define	SC_FMT	0x04	/* ForMaT UNIT */
#define	SC_RD	0x08	/* ReaD */
#define	SC_WR	0x0a	/* WRite */
#define	SC_SK	0x0b	/* SeeK */
#define SC_INQ	0x12	/* INQuiry */
#define SC_MSL	0x15	/* Mode SeLect */
#define SC_MSN	0x1a	/* Mode SeNse */
#define SC_CAP	0x25	/* read CAPacity */

/*
 * SCSI phase status bytes
 */
#define	PS_BUS	0x00	/* Bus Free Phase */
#define	PS_SEL	0x10	/* Selection Phase */
#define	PS_CMD	0x8a	/* Command Phase */
#define	PS_DIN	0x89	/* Data In Phase */
#define	PS_DOT	0x88	/* Data Out Phase */
#define	PS_STT	0x8b	/* Status Phase */
#define	PS_MIN	0x8f	/* Message In Phase */
#define	PS_MOT	0x8e	/* Message Out Phase */

/*
 * bits in completion status byte:
 */
#define	SD_CC	0x02	/* Check Condition */

/*
 * sense key returned by SC_RQS:
 */
#define	NoSense		0x00
#define	RecoveredError	0x01
#define	NotReady      	0x02
#define	UnitAttention	0x06

/*
 * Recover status
 */
#define RETRY 1		/* bit 1-4 is used by status code */
#define ERR   0x20

/*
 * SCSI controller information
 */
#define SCSI_VALID	(*(char *) 0xed006f == 'V')
#define SCSI_IF		((*(char *) 0xed0070) & 8)
#define HOST_ID		((*(char *) 0xed0070) & 7)
#define SASI_FLAG(t)	((*(char *) 0xed0071) & (1 << t))

/*
 * Partition information
 */
struct pi {
  long	pi_name[2];		/* OS name */
  long	pi_start;		/* first sector */
  long	pi_size;		/* total number of sectors */
};

#define	NPARTS		16
#define	D_TYPE(t)	(((unsigned char *) &pi[t << 4])[0])
#define	REMOVE(t)	(((unsigned char *) &pi[t << 4])[1])
#define	ANSI_V(t)	(((unsigned char *) &pi[t << 4])[2])
#define	D_AVAL(t)	(((unsigned char *) &pi[t << 4])[3])

/*
 * Hard disk information (sector 4) of any hard disk drive
 */
struct hi {
  long	hi_name[2];		/* OS name */
  long	hi_start;		/* first sector */
  long	hi_size;		/* total number of sectors */
};

/* values of hi_act: */
#define	PI_BOOT		0x00
#define	PI_VALID	0x02
